@@ -1,13 +1,9 @@
-use crate::data::{BundleSet, ComponentSet, ComponentSetOption};
-use crate::dynamics::{
-    IslandManager, RigidBodyColliders, RigidBodyForces, RigidBodyMassProps, RigidBodyPosition,
-    RigidBodyVelocity,
-};
+use crate::dynamics::IslandManager;
 use crate::geometry::{
-    ColliderFlags, ColliderHandle, ColliderParent, ColliderPosition, ColliderShape,
-    InteractionGroups, PointProjection, Ray, RayIntersection, AABB, QBVH,
+    ColliderHandle, InteractionGroups, PointProjection, Ray, RayIntersection, AABB, QBVH,
 };
 use crate::math::{Isometry, Point, Real, Vector};
+use crate::{dynamics::RigidBodySet, geometry::ColliderSet};
 use parry::partitioning::QBVHDataGenerator;
 use parry::query::details::{
     IntersectionCompositeShapeShapeBestFirstVisitor,
@@ -23,9 +19,6 @@ use parry::query::{DefaultQueryDispatcher, NonlinearRigidMotion, QueryDispatcher
 use parry::shape::{FeatureId, Shape, TypedSimdCompositeShape};
 use std::sync::Arc;
 
-#[cfg(feature = "default-sets")]
-use crate::{dynamics::RigidBodySet, geometry::ColliderSet};
-
 /// A pipeline for performing queries on all the colliders of a scene.
 #[cfg_attr(feature = "serde-serialize", derive(Serialize, Deserialize))]
 #[derive(Clone)]
@@ -40,9 +33,9 @@ pub struct QueryPipeline {
     dilation_factor: Real,
 }
 
-struct QueryPipelineAsCompositeShape<'a, Colliders> {
+struct QueryPipelineAsCompositeShape<'a> {
     query_pipeline: &'a QueryPipeline,
-    colliders: &'a Colliders,
+    colliders: &'a ColliderSet,
     query_groups: InteractionGroups,
     filter: Option<&'a dyn Fn(ColliderHandle) -> bool>,
 }
@@ -63,12 +56,7 @@ pub enum QueryPipelineMode {
     },
 }
 
-impl<'a, Colliders> TypedSimdCompositeShape for QueryPipelineAsCompositeShape<'a, Colliders>
-where
-    // TODO ECS: make everything optional but the shape?
-    Colliders:
-        ComponentSet<ColliderFlags> + ComponentSet<ColliderPosition> + ComponentSet<ColliderShape>,
-{
+impl<'a> TypedSimdCompositeShape for QueryPipelineAsCompositeShape<'a> {
     type PartShape = dyn Shape;
     type PartId = ColliderHandle;
 
@@ -77,15 +65,11 @@ where
         shape_id: Self::PartId,
         mut f: impl FnMut(Option<&Isometry<Real>>, &Self::PartShape),
     ) {
-        let co_flags: Option<&ColliderFlags> = self.colliders.get(shape_id.0);
-
-        if let Some(co_flags) = co_flags {
-            if co_flags.collision_groups.test(self.query_groups)
+        if let Some(co) = self.colliders.get(shape_id) {
+            if co.flags.collision_groups.test(self.query_groups)
                 && self.filter.map(|f| f(shape_id)).unwrap_or(true)
             {
-                let (co_pos, co_shape): (&ColliderPosition, &ColliderShape) =
-                    self.colliders.index_bundle(shape_id.0);
-                f(Some(co_pos), &**co_shape)
+                f(Some(&co.pos), &*co.shape)
             }
         }
     }
@@ -115,12 +99,12 @@ impl QueryPipeline {
         Self::with_query_dispatcher(DefaultQueryDispatcher)
     }
 
-    fn as_composite_shape<'a, Colliders>(
+    fn as_composite_shape<'a>(
         &'a self,
-        colliders: &'a Colliders,
+        colliders: &'a ColliderSet,
         query_groups: InteractionGroups,
         filter: Option<&'a dyn Fn(ColliderHandle) -> bool>,
-    ) -> QueryPipelineAsCompositeShape<'a, Colliders> {
+    ) -> QueryPipelineAsCompositeShape<'a> {
         QueryPipelineAsCompositeShape {
             query_pipeline: self,
             colliders,
@@ -150,7 +134,6 @@ impl QueryPipeline {
         &*self.query_dispatcher
     }
 
-    #[cfg(feature = "default-sets")]
     /// Update the acceleration structure on the query pipeline.
     pub fn update(
         &mut self,
@@ -158,25 +141,6 @@ impl QueryPipeline {
         bodies: &RigidBodySet,
         colliders: &ColliderSet,
     ) {
-        self.update_generic(islands, bodies, colliders);
-    }
-
-    /// Update the acceleration structure on the query pipeline.
-    pub fn update_generic<Bodies, Colliders>(
-        &mut self,
-        islands: &IslandManager,
-        bodies: &Bodies,
-        colliders: &Colliders,
-    ) where
-        Bodies: ComponentSet<RigidBodyPosition>
-            + ComponentSet<RigidBodyColliders>
-            + ComponentSet<RigidBodyVelocity>
-            + ComponentSet<RigidBodyMassProps>
-            + ComponentSet<RigidBodyForces>,
-        Colliders: ComponentSet<ColliderShape>
-            + ComponentSet<ColliderPosition>
-            + ComponentSetOption<ColliderParent>,
-    {
         self.update_with_mode(
             islands,
             bodies,
@@ -186,94 +150,57 @@ impl QueryPipeline {
     }
 
     /// Update the acceleration structure on the query pipeline.
-    pub fn update_with_mode<Bodies, Colliders>(
+    pub fn update_with_mode(
         &mut self,
         islands: &IslandManager,
-        bodies: &Bodies,
-        colliders: &Colliders,
+        bodies: &RigidBodySet,
+        colliders: &ColliderSet,
         mode: QueryPipelineMode,
-    ) where
-        Bodies: ComponentSet<RigidBodyPosition>
-            + ComponentSet<RigidBodyColliders>
-            + ComponentSet<RigidBodyVelocity>
-            + ComponentSet<RigidBodyMassProps>
-            + ComponentSet<RigidBodyForces>,
-        Colliders: ComponentSet<ColliderShape>
-            + ComponentSet<ColliderPosition>
-            + ComponentSetOption<ColliderParent>,
-    {
-        struct DataGenerator<'a, Bs, Cs> {
-            bodies: &'a Bs,
-            colliders: &'a Cs,
+    ) {
+        struct DataGenerator<'a> {
+            bodies: &'a RigidBodySet,
+            colliders: &'a ColliderSet,
             mode: QueryPipelineMode,
         }
 
-        impl<'a, Bs, Cs> QBVHDataGenerator<ColliderHandle> for DataGenerator<'a, Bs, Cs>
-        where
-            Bs: ComponentSet<RigidBodyPosition>
-                + ComponentSet<RigidBodyMassProps>
-                + ComponentSet<RigidBodyVelocity>
-                + ComponentSet<RigidBodyForces>,
-            Cs: ComponentSet<ColliderShape>
-                + ComponentSet<ColliderPosition>
-                + ComponentSetOption<ColliderParent>,
-        {
+        impl<'a> QBVHDataGenerator<ColliderHandle> for DataGenerator<'a> {
             fn size_hint(&self) -> usize {
-                ComponentSet::<ColliderShape>::size_hint(self.colliders)
+                self.colliders.len()
             }
 
             #[inline(always)]
             fn for_each(&mut self, mut f: impl FnMut(ColliderHandle, AABB)) {
                 match self.mode {
                     QueryPipelineMode::CurrentPosition => {
-                        self.colliders.for_each(|h, co_shape: &ColliderShape| {
-                            let co_pos: &ColliderPosition = self.colliders.index(h);
-                            f(ColliderHandle(h), co_shape.compute_aabb(&co_pos))
-                        })
+                        for (h, co) in self.colliders.iter() {
+                            f(h, co.shape.compute_aabb(&co.pos))
+                        }
                     }
                     QueryPipelineMode::SweepTestWithNextPosition => {
-                        self.colliders.for_each(|h, co_shape: &ColliderShape| {
-                            let co_parent: Option<&ColliderParent> = self.colliders.get(h);
-                            let co_pos: &ColliderPosition = self.colliders.index(h);
-
-                            if let Some(co_parent) = co_parent {
-                                let rb_pos: &RigidBodyPosition =
-                                    self.bodies.index(co_parent.handle.0);
-                                let next_position = rb_pos.next_position * co_parent.pos_wrt_parent;
-
-                                f(
-                                    ColliderHandle(h),
-                                    co_shape.compute_swept_aabb(&co_pos, &next_position),
-                                )
+                        for (h, co) in self.colliders.iter() {
+                            if let Some(co_parent) = co.parent {
+                                let rb_next_pos = &self.bodies[co_parent.handle].pos.next_position;
+                                let next_position = rb_next_pos * co_parent.pos_wrt_parent;
+                                f(h, co.shape.compute_swept_aabb(&co.pos, &next_position))
                             } else {
-                                f(ColliderHandle(h), co_shape.compute_aabb(&co_pos))
+                                f(h, co.shape.compute_aabb(&co.pos))
                             }
-                        })
+                        }
                     }
                     QueryPipelineMode::SweepTestWithPredictedPosition { dt } => {
-                        self.colliders.for_each(|h, co_shape: &ColliderShape| {
-                            let co_parent: Option<&ColliderParent> = self.colliders.get(h);
-                            let co_pos: &ColliderPosition = self.colliders.index(h);
-
-                            if let Some(co_parent) = co_parent {
-                                let (rb_pos, vels, forces, mprops): (
-                                    &RigidBodyPosition,
-                                    &RigidBodyVelocity,
-                                    &RigidBodyForces,
-                                    &RigidBodyMassProps,
-                                ) = self.bodies.index_bundle(co_parent.handle.0);
-                                let predicted_pos = rb_pos
-                                    .integrate_forces_and_velocities(dt, forces, vels, mprops);
+                        for (h, co) in self.colliders.iter() {
+                            if let Some(co_parent) = co.parent {
+                                let rb = &self.bodies[co_parent.handle];
+                                let predicted_pos = rb.pos.integrate_forces_and_velocities(
+                                    dt, &rb.forces, &rb.vels, &rb.mprops,
+                                );
 
                                 let next_position = predicted_pos * co_parent.pos_wrt_parent;
-                                f(
-                                    ColliderHandle(h),
-                                    co_shape.compute_swept_aabb(&co_pos, &next_position),
-                                )
+                                f(h, co.shape.compute_swept_aabb(&co.pos, &next_position))
                             } else {
-                                f(ColliderHandle(h), co_shape.compute_aabb(&co_pos))
+                                f(h, co.shape.compute_aabb(&co.pos))
                             }
-                        })
+                        }
                     }
                 }
             }
@@ -293,8 +220,8 @@ impl QueryPipeline {
         }
 
         for handle in islands.iter_active_bodies() {
-            let body_colliders: &RigidBodyColliders = bodies.index(handle.0);
-            for handle in &body_colliders.0 {
+            let rb = &bodies[handle];
+            for handle in &rb.colliders.0 {
                 self.qbvh.pre_update(*handle)
             }
         }
@@ -303,9 +230,8 @@ impl QueryPipeline {
             QueryPipelineMode::CurrentPosition => {
                 self.qbvh.update(
                     |handle| {
-                        let (co_pos, co_shape): (&ColliderPosition, &ColliderShape) =
-                            colliders.index_bundle(handle.0);
-                        co_shape.compute_aabb(&co_pos)
+                        let co = &colliders[*handle];
+                        co.shape.compute_aabb(&co.pos)
                     },
                     self.dilation_factor,
                 );
@@ -313,16 +239,13 @@ impl QueryPipeline {
             QueryPipelineMode::SweepTestWithNextPosition => {
                 self.qbvh.update(
                     |handle| {
-                        let co_parent: Option<&ColliderParent> = colliders.get(handle.0);
-                        let (co_pos, co_shape): (&ColliderPosition, &ColliderShape) =
-                            colliders.index_bundle(handle.0);
-
-                        if let Some(co_parent) = co_parent {
-                            let rb_pos: &RigidBodyPosition = bodies.index(co_parent.handle.0);
-                            let next_position = rb_pos.next_position * co_parent.pos_wrt_parent;
-                            co_shape.compute_swept_aabb(&co_pos, &next_position)
+                        let co = &colliders[*handle];
+                        if let Some(parent) = &co.parent {
+                            let rb_next_pos = &bodies[parent.handle].pos.next_position;
+                            let next_position = rb_next_pos * parent.pos_wrt_parent;
+                            co.shape.compute_swept_aabb(&co.pos, &next_position)
                         } else {
-                            co_shape.compute_aabb(&co_pos)
+                            co.shape.compute_aabb(&co.pos)
                         }
                     },
                     self.dilation_factor,
@@ -331,25 +254,17 @@ impl QueryPipeline {
             QueryPipelineMode::SweepTestWithPredictedPosition { dt } => {
                 self.qbvh.update(
                     |handle| {
-                        let co_parent: Option<&ColliderParent> = colliders.get(handle.0);
-                        let (co_pos, co_shape): (&ColliderPosition, &ColliderShape) =
-                            colliders.index_bundle(handle.0);
+                        let co = &colliders[*handle];
+                        if let Some(parent) = co.parent {
+                            let rb = &bodies[parent.handle];
+                            let predicted_pos = rb.pos.integrate_forces_and_velocities(
+                                dt, &rb.forces, &rb.vels, &rb.mprops,
+                            );
 
-                        if let Some(co_parent) = co_parent {
-                            let (rb_pos, vels, forces, mprops): (
-                                &RigidBodyPosition,
-                                &RigidBodyVelocity,
-                                &RigidBodyForces,
-                                &RigidBodyMassProps,
-                            ) = bodies.index_bundle(co_parent.handle.0);
-
-                            let predicted_pos =
-                                rb_pos.integrate_forces_and_velocities(dt, forces, vels, mprops);
-
-                            let next_position = predicted_pos * co_parent.pos_wrt_parent;
-                            co_shape.compute_swept_aabb(&co_pos, &next_position)
+                            let next_position = predicted_pos * parent.pos_wrt_parent;
+                            co.shape.compute_swept_aabb(&co.pos, &next_position)
                         } else {
-                            co_shape.compute_aabb(&co_pos)
+                            co.shape.compute_aabb(&co.pos)
                         }
                     },
                     self.dilation_factor,
@@ -361,32 +276,27 @@ impl QueryPipeline {
     /// Find the closest intersection between a ray and a set of collider.
     ///
     /// # Parameters
-    /// - `position`: the position of this shape.
-    /// - `ray`: the ray to cast.
-    /// - `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
+    /// * `colliders` - The set of colliders taking part in this pipeline.
+    /// * `ray`: the ray to cast.
+    /// * `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
     ///   limits the length of the ray to `ray.dir.norm() * max_toi`. Use `Real::MAX` for an unbounded ray.
-    /// - `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
+    /// * `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
     ///            it starts inside of a shape. If this `false` then the ray will hit the shape's boundary
     ///            even if its starts inside of it.
-    /// - `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
+    /// * `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
     ///                   to determine if it should be taken into account by this query.
-    /// - `filter`: a more fine-grained filter. A collider is taken into account by this query if
+    /// * `filter`: a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn cast_ray<Colliders>(
+    pub fn cast_ray(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         ray: &Ray,
         max_toi: Real,
         solid: bool,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, Real)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, Real)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor =
             RayCompositeShapeToiBestFirstVisitor::new(&pipeline_shape, ray, max_toi, solid);
@@ -397,32 +307,27 @@ impl QueryPipeline {
     /// Find the closest intersection between a ray and a set of collider.
     ///
     /// # Parameters
-    /// - `position`: the position of this shape.
-    /// - `ray`: the ray to cast.
-    /// - `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
+    /// * `colliders` - The set of colliders taking part in this pipeline.
+    /// * `ray`: the ray to cast.
+    /// * `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
     ///   limits the length of the ray to `ray.dir.norm() * max_toi`. Use `Real::MAX` for an unbounded ray.
-    /// - `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
+    /// * `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
     ///            it starts inside of a shape. If this `false` then the ray will hit the shape's boundary
     ///            even if its starts inside of it.
-    /// - `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
+    /// * `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
     ///                   to determine if it should be taken into account by this query.
-    /// - `filter`: a more fine-grained filter. A collider is taken into account by this query if
+    /// * `filter`: a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn cast_ray_and_get_normal<Colliders>(
+    pub fn cast_ray_and_get_normal(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         ray: &Ray,
         max_toi: Real,
         solid: bool,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, RayIntersection)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, RayIntersection)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor = RayCompositeShapeToiAndNormalBestFirstVisitor::new(
             &pipeline_shape,
@@ -437,44 +342,39 @@ impl QueryPipeline {
     /// Find the all intersections between a ray and a set of collider and passes them to a callback.
     ///
     /// # Parameters
-    /// - `position`: the position of this shape.
-    /// - `ray`: the ray to cast.
-    /// - `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
+    /// * `colliders` - The set of colliders taking part in this pipeline.
+    /// * `ray`: the ray to cast.
+    /// * `max_toi`: the maximum time-of-impact that can be reported by this cast. This effectively
     ///   limits the length of the ray to `ray.dir.norm() * max_toi`. Use `Real::MAX` for an unbounded ray.
-    /// - `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
+    /// * `solid`: if this is `true` an impact at time 0.0 (i.e. at the ray origin) is returned if
     ///            it starts inside of a shape. If this `false` then the ray will hit the shape's boundary
     ///            even if its starts inside of it.
-    /// - `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
+    /// * `query_groups`: the interaction groups which will be tested against the collider's `contact_group`
     ///                   to determine if it should be taken into account by this query.
-    /// - `filter`: a more fine-grained filter. A collider is taken into account by this query if
+    /// * `filter`: a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    /// - `callback`: function executed on each collider for which a ray intersection has been found.
+    /// * `callback`: function executed on each collider for which a ray intersection has been found.
     ///               There is no guarantees on the order the results will be yielded. If this callback returns `false`,
     ///               this method will exit early, ignore any further raycast.
-    pub fn intersections_with_ray<'a, Colliders>(
+    pub fn intersections_with_ray<'a>(
         &self,
-        colliders: &'a Colliders,
+        colliders: &'a ColliderSet,
         ray: &Ray,
         max_toi: Real,
         solid: bool,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
         mut callback: impl FnMut(ColliderHandle, RayIntersection) -> bool,
-    ) where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) {
         let mut leaf_callback = &mut |handle: &ColliderHandle| {
-            let co_shape: Option<&ColliderShape> = colliders.get(handle.0);
-            if let Some(co_shape) = co_shape {
-                let (co_flags, co_pos): (&ColliderFlags, &ColliderPosition) =
-                    colliders.index_bundle(handle.0);
-                if co_flags.collision_groups.test(query_groups)
+            if let Some(co) = colliders.get(*handle) {
+                if co.flags.collision_groups.test(query_groups)
                     && filter.map(|f| f(*handle)).unwrap_or(true)
                 {
-                    if let Some(hit) = co_shape.cast_ray_and_get_normal(co_pos, ray, max_toi, solid)
+                    if let Some(hit) = co
+                        .shape
+                        .cast_ray_and_get_normal(&co.pos, ray, max_toi, solid)
                     {
                         return callback(*handle, hit);
                     }
@@ -499,19 +399,14 @@ impl QueryPipeline {
     /// * `filter` - a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn intersection_with_shape<Colliders>(
+    pub fn intersection_with_shape(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         shape_pos: &Isometry<Real>,
         shape: &dyn Shape,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<ColliderHandle>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<ColliderHandle> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor = IntersectionCompositeShapeShapeBestFirstVisitor::new(
             &*self.query_dispatcher,
@@ -540,19 +435,14 @@ impl QueryPipeline {
     /// * `filter` - a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn project_point<Colliders>(
+    pub fn project_point(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         point: &Point<Real>,
         solid: bool,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, PointProjection)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, PointProjection)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor =
             PointCompositeShapeProjBestFirstVisitor::new(&pipeline_shape, point, solid);
@@ -574,28 +464,19 @@ impl QueryPipeline {
     ///             is either `None` or returns `true`.
     /// * `callback` - A function called with each collider with a shape
     ///                containing the `point`.
-    pub fn intersections_with_point<'a, Colliders>(
+    pub fn intersections_with_point<'a>(
         &self,
-        colliders: &'a Colliders,
+        colliders: &'a ColliderSet,
         point: &Point<Real>,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
         mut callback: impl FnMut(ColliderHandle) -> bool,
-    ) where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) {
         let mut leaf_callback = &mut |handle: &ColliderHandle| {
-            let co_shape: Option<&ColliderShape> = colliders.get(handle.0);
-
-            if let Some(co_shape) = co_shape {
-                let (co_flags, co_pos): (&ColliderFlags, &ColliderPosition) =
-                    colliders.index_bundle(handle.0);
-
-                if co_flags.collision_groups.test(query_groups)
+            if let Some(co) = colliders.get(*handle) {
+                if co.flags.collision_groups.test(query_groups)
                     && filter.map(|f| f(*handle)).unwrap_or(true)
-                    && co_shape.contains_point(co_pos, point)
+                    && co.shape.contains_point(&co.pos, point)
                 {
                     return callback(*handle);
                 }
@@ -626,18 +507,13 @@ impl QueryPipeline {
     /// * `filter` - a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn project_point_and_get_feature<Colliders>(
+    pub fn project_point_and_get_feature(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         point: &Point<Real>,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, PointProjection, FeatureId)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, PointProjection, FeatureId)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor =
             PointCompositeShapeProjWithFeatureBestFirstVisitor::new(&pipeline_shape, point, false);
@@ -674,21 +550,16 @@ impl QueryPipeline {
     /// * `filter` - a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn cast_shape<'a, Colliders>(
+    pub fn cast_shape<'a>(
         &self,
-        colliders: &'a Colliders,
+        colliders: &'a ColliderSet,
         shape_pos: &Isometry<Real>,
         shape_vel: &Vector<Real>,
         shape: &dyn Shape,
         max_toi: Real,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, TOI)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, TOI)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let mut visitor = TOICompositeShapeShapeBestFirstVisitor::new(
             &*self.query_dispatcher,
@@ -724,9 +595,9 @@ impl QueryPipeline {
     /// * `filter` - a more fine-grained filter. A collider is taken into account by this query if
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
-    pub fn nonlinear_cast_shape<Colliders>(
+    pub fn nonlinear_cast_shape(
         &self,
-        colliders: &Colliders,
+        colliders: &ColliderSet,
         shape_motion: &NonlinearRigidMotion,
         shape: &dyn Shape,
         start_time: Real,
@@ -734,12 +605,7 @@ impl QueryPipeline {
         stop_at_penetration: bool,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
-    ) -> Option<(ColliderHandle, TOI)>
-    where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) -> Option<(ColliderHandle, TOI)> {
         let pipeline_shape = self.as_composite_shape(colliders, query_groups, filter);
         let pipeline_motion = NonlinearRigidMotion::identity();
         let mut visitor = NonlinearTOICompositeShapeShapeBestFirstVisitor::new(
@@ -768,35 +634,26 @@ impl QueryPipeline {
     ///             its `contact_group` is compatible with the `query_groups`, and if this `filter`
     ///             is either `None` or returns `true`.
     /// * `callback` - A function called with the handles of each collider intersecting the `shape`.
-    pub fn intersections_with_shape<'a, Colliders>(
+    pub fn intersections_with_shape<'a>(
         &self,
-        colliders: &'a Colliders,
+        colliders: &'a ColliderSet,
         shape_pos: &Isometry<Real>,
         shape: &dyn Shape,
         query_groups: InteractionGroups,
         filter: Option<&dyn Fn(ColliderHandle) -> bool>,
         mut callback: impl FnMut(ColliderHandle) -> bool,
-    ) where
-        Colliders: ComponentSet<ColliderFlags>
-            + ComponentSet<ColliderPosition>
-            + ComponentSet<ColliderShape>,
-    {
+    ) {
         let dispatcher = &*self.query_dispatcher;
         let inv_shape_pos = shape_pos.inverse();
 
         let mut leaf_callback = &mut |handle: &ColliderHandle| {
-            let co_shape: Option<&ColliderShape> = colliders.get(handle.0);
-
-            if let Some(co_shape) = co_shape {
-                let (co_flags, co_pos): (&ColliderFlags, &ColliderPosition) =
-                    colliders.index_bundle(handle.0);
-
-                if co_flags.collision_groups.test(query_groups)
+            if let Some(co) = colliders.get(*handle) {
+                if co.flags.collision_groups.test(query_groups)
                     && filter.map(|f| f(*handle)).unwrap_or(true)
                 {
-                    let pos12 = inv_shape_pos * co_pos.as_ref();
+                    let pos12 = inv_shape_pos * co.pos.as_ref();
 
-                    if dispatcher.intersection_test(&pos12, shape, &**co_shape) == Ok(true) {
+                    if dispatcher.intersection_test(&pos12, shape, &*co.shape) == Ok(true) {
                         return callback(*handle);
                     }
                 }
